@@ -1,30 +1,30 @@
-/****************************************************************************/
-/*                                                                          */
-/*                         GNAT COMPILER COMPONENTS                         */
-/*                                                                          */
-/*                              A _ A T R E E                               */
-/*                                                                          */
-/*                              C Header File                               */
-/*                                                                          */
-/*                            $Revision: 1.62 $
-/*                                                                          */
-/*          Copyright (C) 1992-2000, Free Software Foundation, Inc.         */
-/*                                                                          */
-/* GNAT is free software;  you can  redistribute it  and/or modify it under */
-/* terms of the  GNU General Public License as published  by the Free Soft- */
-/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
-/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
-/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
-/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
-/* for  more details.  You should have  received  a copy of the GNU General */
-/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
-/* MA 02111-1307, USA.                                                      */
-/*                                                                          */
-/* GNAT was originally developed  by the GNAT team at  New York University. */
-/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
-/*                                                                          */
-/****************************************************************************/
+/****************************************************************************
+ *                                                                          *
+ *                         GNAT COMPILER COMPONENTS                         *
+ *                                                                          *
+ *                              A _ A T R E E                               *
+ *                                                                          *
+ *                              C Header File                               *
+ *                                                                          *
+ *                            $Revision: 1.65 $
+ *                                                                          *
+ *          Copyright (C) 1992-2001, Free Software Foundation, Inc.         *
+ *                                                                          *
+ * GNAT is free software;  you can  redistribute it  and/or modify it under *
+ * terms of the  GNU General Public License as published  by the Free Soft- *
+ * ware  Foundation;  either version 2,  or (at your option) any later ver- *
+ * sion.  GNAT is distributed in the hope that it will be useful, but WITH- *
+ * OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY *
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
+ * for  more details.  You should have  received  a copy of the GNU General *
+ * Public License  distributed with GNAT;  see file COPYING.  If not, write *
+ * to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, *
+ * MA 02111-1307, USA.                                                      *
+ *                                                                          *
+ * GNAT was originally developed  by the GNAT team at  New York University. *
+ * It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). *
+ *                                                                          *
+ ****************************************************************************/
 
 /* This is the C header corresponding to the Ada package specification for
    Atree. It also contains the implementations of inlined functions from the
@@ -237,7 +237,7 @@ struct Extended
   Int	       field8;
   Int	       field9;
   Int	       field10;
-  union     
+  union
     {
       Int      field11;
       struct Flag_Word3 fw3;
@@ -274,7 +274,6 @@ struct Node
    that Node_Id values can be used as subscripts.  */
 extern struct Node *Nodes_Ptr;
 
-
 #define Parent atree__parent
 extern Node_Id Parent PROTO((Node_Id));
 
@@ -393,6 +392,7 @@ extern Node_Id Current_Error_Node;
 #define Uint5(N)      ((Field5  (N)==0) ? Uint_0 : Field5  (N))
 #define Uint8(N)      ((Field8  (N)==0) ? Uint_0 : Field8  (N))
 #define Uint9(N)      ((Field9  (N)==0) ? Uint_0 : Field9  (N))
+#define Uint10(N)     ((Field10 (N)==0) ? Uint_0 : Field10 (N))
 #define Uint11(N)     ((Field11 (N)==0) ? Uint_0 : Field11 (N))
 #define Uint12(N)     ((Field12 (N)==0) ? Uint_0 : Field12 (N))
 #define Uint13(N)     ((Field13 (N)==0) ? Uint_0 : Field13 (N))
