@@ -1,36 +1,36 @@
-/****************************************************************************/
-/*                                                                          */
-/*                         GNAT COMPILER COMPONENTS                         */
-/*                                                                          */
-/*                               A - M I S C                                */
-/*                                                                          */
-/*                           C Implementation File                          */
-/*                                                                          */
-/*                             $Revision: 1.169 $
-/*                                                                          */
-/*          Copyright (C) 1992-2000 Free Software Foundation, Inc.          */
-/*                                                                          */
-/* GNAT is free software;  you can  redistribute it  and/or modify it under */
-/* terms of the  GNU General Public License as published  by the Free Soft- */
-/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
-/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
-/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
-/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
-/* for  more details.  You should have  received  a copy of the GNU General */
-/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
-/* MA 02111-1307, USA.                                                      */
-/*                                                                          */
-/* As a  special  exception,  if you  link  this file  with other  files to */
-/* produce an executable,  this file does not by itself cause the resulting */
-/* executable to be covered by the GNU General Public License. This except- */
-/* ion does not  however invalidate  any other reasons  why the  executable */
-/* file might be covered by the  GNU Public License.                        */
-/*                                                                          */
-/* GNAT was originally developed  by the GNAT team at  New York University. */
-/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
-/*                                                                          */
-/****************************************************************************/
+/****************************************************************************
+ *                                                                          *
+ *                         GNAT COMPILER COMPONENTS                         *
+ *                                                                          *
+ *                               A - M I S C                                *
+ *                                                                          *
+ *                           C Implementation File                          *
+ *                                                                          *
+ *                             $Revision: 1.175.2.1 $
+ *                                                                          *
+ *          Copyright (C) 1992-2002 Free Software Foundation, Inc.          *
+ *                                                                          *
+ * GNAT is free software;  you can  redistribute it  and/or modify it under *
+ * terms of the  GNU General Public License as published  by the Free Soft- *
+ * ware  Foundation;  either version 2,  or (at your option) any later ver- *
+ * sion.  GNAT is distributed in the hope that it will be useful, but WITH- *
+ * OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY *
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
+ * for  more details.  You should have  received  a copy of the GNU General *
+ * Public License  distributed with GNAT;  see file COPYING.  If not, write *
+ * to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, *
+ * MA 02111-1307, USA.                                                      *
+ *                                                                          *
+ * As a  special  exception,  if you  link  this file  with other  files to *
+ * produce an executable,  this file does not by itself cause the resulting *
+ * executable to be covered by the GNU General Public License. This except- *
+ * ion does not  however invalidate  any other reasons  why the  executable *
+ * file might be covered by the  GNU Public License.                        *
+ *                                                                          *
+ * GNAT was originally developed  by the GNAT team at  New York University. *
+ * It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). *
+ *                                                                          *
+ ****************************************************************************/
 
 /* This unit needs a global comment saying what is in it ??? */
 
@@ -68,7 +68,7 @@ extern FILE *asm_out_file;
 extern int save_argc;
 extern char **save_argv;
 
-/* Tables describing GCC tree codes used only by GNAT.  
+/* Tables describing GCC tree codes used only by GNAT.
 
    Table indexed by tree code giving a string containing a character
    classifying the tree code.  Possibilities are
@@ -118,7 +118,6 @@ int flag_traditional;		/* Used by dwarfout.c.  */
 static void fatal_function	PROTO((char *, va_list));
 static rtx gnat_expand_expr	PROTO((tree, rtx, enum machine_mode,
 				       enum expand_modifier));
-static char *convert_ada_name_to_qualified_name PROTO((char *));
 
 /* Routines Expected by gcc:  */
 
@@ -164,13 +163,13 @@ yyparse ()
 
 /* init gnat_argc and gnat_argv */
 
-void 
+void
 init_gnat_args ()
 {
   /* Initialize gnat_argv with save_argv size */
-  gnat_argv = (char **) xmalloc ((save_argc + 1) * sizeof (gnat_argv[0])); 
+  gnat_argv = (char **) xmalloc ((save_argc + 1) * sizeof (gnat_argv[0]));
 
-  gnat_argv [0] = save_argv[0];     /* name of the command */ 
+  gnat_argv [0] = save_argv[0];     /* name of the command */
   gnat_argc = 1;
 }
 
@@ -215,7 +214,7 @@ lang_decode_option (p)
       strcpy (gnat_argv[gnat_argc] + 1, p + 5);
       gnat_argc ++;
       if (p[5] == 'O')
-	for (i = 1; i < save_argc - 1; i++) 
+	for (i = 1; i < save_argc - 1; i++)
 	  if (!strncmp (save_argv[i], "-gnatO", 6))
 	    if (save_argv[++i][0] != '-')
 	      {
@@ -230,6 +229,15 @@ lang_decode_option (p)
       return 1;
     }
 
+  /* handle the --RTS switch
+     The real option we get is -fRTS. This modification is done by gcc */
+  if (!strncmp (p, "-fRTS", 5))
+    {
+      gnat_argv[gnat_argc] = p;
+      gnat_argc ++;
+      return 1;
+    }
+
   return 0;
 }
 
@@ -294,10 +302,6 @@ lang_init ()
 #ifdef DO_FATAL_FUNCTION
   set_fatal_function (fatal_function);
 #endif
-
-#if defined(SGI_TARGET) && defined(DWARF2_DEBUGGING_INFO)
-  dwarf2out_set_demangle_name_func (convert_ada_name_to_qualified_name);
-#endif
 }
 
 /* Perform all the finalization steps that are language-specific.  */
@@ -494,7 +498,14 @@ gnat_expand_expr (exp, target, tmode, modifier)
 	      if (TREE_CODE_CLASS (TREE_CODE (new)) == 'd')
 		new = build1 (NOP_EXPR, copy_type, new);
 	      else
-		TREE_TYPE (new) = copy_type;
+		{
+		  /* If NEW is a constant, it might be coming from a CONST_DECL
+		     and hence shared.  */
+		  if (TREE_CONSTANT (new))
+		    new = copy_node (new);
+
+		  TREE_TYPE (new) = copy_type;
+		}
 	    }
 	}
 
@@ -776,9 +787,16 @@ insert_code_for (gnat_node)
     {
       rtx insns;
 
+      /* Issue pending stack adjustments to make the code self contained.
+	 Avoids troubles such as having a call inserted before a dynamic stack
+	 allocation and the associated pops deferred until after this
+	 allocation.  */
+
+      do_pending_stack_adjust ();
       start_sequence ();
       mark_all_temps_used ();
       gnat_to_code (gnat_node);
+      do_pending_stack_adjust ();
       insns = get_insns ();
       end_sequence ();
       emit_insns_after (insns, (rtx) get_gnu_tree (gnat_node));
@@ -829,7 +847,6 @@ set_yydebug (value)
      int value;
 {}
 
-
 /* Override the regular abort to call gigi_abort since it gives more useful
    crash error messages.  If abort is a macro, we can't do this.  */
 
@@ -899,110 +916,3 @@ must_pass_by_ref (gnu_type)
 	  || (TYPE_SIZE (gnu_type) != 0
 	      && TREE_CODE (TYPE_SIZE (gnu_type)) != INTEGER_CST));
 }
-
-/* Convert NAME, which is possibly an Ada name, back to standard Ada
-   notation for SGI Workshop.  Return the new name. This returns NAME
-   if we aren't compiling for SGI.  */
-
-static char *
-convert_ada_name_to_qualified_name (name)
-     char *name;
-{
-#ifndef SGI_TARGET
-  return name;
-#else
-
-  int len = strlen (name);
-  char *new_name = xstrdup (name);
-  char *buf;
-  int i, start;
-  char *qual_name_suffix = 0;
-  char *p;
-
-  if (len <= 3 || use_gnu_debug_info_extensions)
-    {
-      free (new_name);
-      return name;
-    }
-
-  /* Find the position of the first "__" after the first character of
-     NAME.  This is the same as calling strstr except that we can't assume
-     the host has that function. We start after the first character so
-     we don't eliminate leading "__": these are emitted only by C
-     programs and are not qualified names */
-  for (p = (char *) index (&name[1], '_'); p != 0;
-       p = (char *) index (p+1, '_'))
-    if (p[1] == '_')
-      {
-	qual_name_suffix = p;
-	break;
-      }
-
-  if (qual_name_suffix == 0)
-    {
-      free (new_name);
-      return name;
-    }
-
-  start = qual_name_suffix - name;
-  buf = new_name + start;
-
-  for (i = start; i < len; i++)
-    {
-      if (name[i] == '_' && name[i + 1] == '_')
-	{
-	  if (islower (name[i + 2]))
-	    {
-	      *buf++ = '.';
-	      *buf++ = name[i + 2];
-	      i += 2;
-	    }
-	  else if (name[i + 2] == '_' && islower (name[i + 3]))
-	    { 
-	      /* convert foo___c___XVN to foo.c___XVN */
-	      *buf++ = '.';
-	      *buf++ = name[i + 3];
-	      i += 3;
-	    }
-	  else if (name[i + 2] == 'T')
-	    {
-	      /* convert foo__TtypeS to foo.__TTypeS */
-	      *buf++ = '.';
-	      *buf++ = '_';
-	      *buf++ = '_';
-	      *buf++ = 'T';
-	      i += 3;
-	    }
-	  else
-	    *buf++ = name[i];
-	}
-      else
-	*buf++ = name[i];
-    }
-
-  *buf = 0;
-  return new_name;
-#endif
-}
-
-/* Emit a label UNITNAME_LABEL and specify that it is part of source
-   file FILENAME.  If this is being written for SGI's Workshop
-   debugger, and we are writing Dwarf2 debugging information, add
-   additional debug info.  */
-
-void
-emit_unit_label (unitname_label, filename)
-     char *unitname_label;
-     char *filename;
-{
-  ASM_GLOBALIZE_LABEL (asm_out_file, unitname_label);
-  ASM_OUTPUT_LABEL (asm_out_file, unitname_label); 
-
-#ifdef SGI_TARGET
-#ifdef DWARF2_DEBUGGING_INFO
-  if (write_symbols == DWARF2_DEBUG)
-    dwarf2out_add_library_unit_info
-      (filename, convert_ada_name_to_qualified_name (unitname_label));
-#endif
-#endif
-}
