@@ -1,55 +1,59 @@
-/****************************************************************************/
-/*                                                                          */
-/*                         GNAT COMPILER COMPONENTS                         */
-/*                                                                          */
-/*                              A - T R A N S                               */
-/*                                                                          */
-/*                              C Header File                               */
-/*                                                                          */
-/*                            $Revision: 1.50 $
-/*                                                                          */
-/*          Copyright (C) 1992-2000 Free Software Foundation, Inc.          */
-/*                                                                          */
-/* GNAT is free software;  you can  redistribute it  and/or modify it under */
-/* terms of the  GNU General Public License as published  by the Free Soft- */
-/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
-/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
-/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
-/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
-/* for  more details.  You should have  received  a copy of the GNU General */
-/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
-/* MA 02111-1307, USA.                                                      */
-/*                                                                          */
-/* GNAT was originally developed  by the GNAT team at  New York University. */
-/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
-/*                                                                          */
-/****************************************************************************/
+/****************************************************************************
+ *                                                                          *
+ *                         GNAT COMPILER COMPONENTS                         *
+ *                                                                          *
+ *                              A - T R A N S                               *
+ *                                                                          *
+ *                              C Header File                               *
+ *                                                                          *
+ *                            $Revision: 1.54 $
+ *                                                                          *
+ *          Copyright (C) 1992-2002 Free Software Foundation, Inc.          *
+ *                                                                          *
+ * GNAT is free software;  you can  redistribute it  and/or modify it under *
+ * terms of the  GNU General Public License as published  by the Free Soft- *
+ * ware  Foundation;  either version 2,  or (at your option) any later ver- *
+ * sion.  GNAT is distributed in the hope that it will be useful, but WITH- *
+ * OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY *
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
+ * for  more details.  You should have  received  a copy of the GNU General *
+ * Public License  distributed with GNAT;  see file COPYING.  If not, write *
+ * to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, *
+ * MA 02111-1307, USA.                                                      *
+ *                                                                          *
+ * GNAT was originally developed  by the GNAT team at  New York University. *
+ * It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). *
+ *                                                                          *
+ ****************************************************************************/
 
 #define BAD_GNAT_TREE ((Node_Id) -1)
 
+/* Test if the node N is the name of an entity (i.e. is an identifier,
+   expanded name, or an attribute reference that returns an entity).  */
+#define Is_Entity_Name einfo__is_entity_name
+extern Boolean Is_Entity_Name PROTO((Node_Id));
+
 /* This function isn't in sinfo.h since we don't make the setting functions,
    just the retrieval functions.  This is the single exception to the rule
    that Gigi doesn't modify the tree.  */
 #define Set_Has_No_Elaboration_Code sinfo__set_has_no_elaboration_code
 extern void Set_Has_No_Elaboration_Code	PROTO((Node_Id, Boolean));
 
-/* We will be setting Esize for types, Component_First_Bit for fields,
+/* We will be setting Esize for types, Component_Bit_Offset for fields,
    Alignment for types and objects, Component_Size for array types, and
    Present_Expr for N_Variant nodes.  */
 #define Set_Alignment		einfo__set_alignment
 #define Set_Esize		einfo__set_esize
 #define Set_RM_Size		einfo__set_rm_size
-#define Set_Component_First_Bit einfo__set_component_first_bit
+#define Set_Component_Bit_Offset einfo__set_component_bit_offset
 #define Set_Component_Size	einfo__set_component_size
 #define Set_Present_Expr	sinfo__set_present_expr
 
 extern void Set_Component_Size		PROTO((Entity_Id, Uint));
 extern void Set_Esize			PROTO((Entity_Id, Uint));
-extern void Set_Component_First_Bit	PROTO((Entity_Id, Uint));
+extern void Set_Component_Bit_Offset	PROTO((Entity_Id, Uint));
 extern void Set_Present_Expr		PROTO((Node_Id, Uint));
 
-
 /* These are definitions from exp_code.ads. */
 #define Is_Asm_Volatile exp_code__is_asm_volatile
 extern Boolean Is_Asm_Volatile	PROTO((Node_Id));
@@ -118,6 +122,10 @@ extern Boolean Debug_Flag_XX, Debug_Flag_NN;
 #define Stack_Check_Probes_On_Target targparm__stack_check_probes_on_target
 extern Boolean Stack_Check_Probes_On_Target;
 
+/* Definitions from restrict.ads */
+#define No_Exception_Handlers_Set restrict__no_exception_handlers_set
+extern Boolean No_Exception_Handlers_Set PROTO((void));
+
 /* Flag indicating whether file names are discarded in exception messages */
 extern int discard_file_names;
 
@@ -207,7 +215,7 @@ extern void init_code_table	PROTO((void));
 extern Node_Id error_gnat_node;
 
 /* This is equivalent to stabilize_reference in GCC's tree.c, but we know
-   how to handle our new nodes and we take an extra argument that says 
+   how to handle our new nodes and we take an extra argument that says
    whether to force evaluation of everything.  */
 
 extern tree gnat_stabilize_reference PROTO((tree, int));
