@@ -1,30 +1,30 @@
-/****************************************************************************/
-/*                                                                          */
-/*                         GNAT COMPILER COMPONENTS                         */
-/*                                                                          */
-/*                             A - T R A N S 3                              */
-/*                                                                          */
-/*                              C Header File                               */
-/*                                                                          */
-/*                            $Revision: 1.126 $
-/*                                                                          */
-/*          Copyright (C) 1992-2000 Free Software Foundation, Inc.          */
-/*                                                                          */
-/* GNAT is free software;  you can  redistribute it  and/or modify it under */
-/* terms of the  GNU General Public License as published  by the Free Soft- */
-/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
-/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
-/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
-/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
-/* for  more details.  You should have  received  a copy of the GNU General */
-/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
-/* MA 02111-1307, USA.                                                      */
-/*                                                                          */
-/* GNAT was originally developed  by the GNAT team at  New York University. */
-/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
-/*                                                                          */
-/****************************************************************************/
+/****************************************************************************
+ *                                                                          *
+ *                         GNAT COMPILER COMPONENTS                         *
+ *                                                                          *
+ *                             A - T R A N S 3                              *
+ *                                                                          *
+ *                              C Header File                               *
+ *                                                                          *
+ *                            $Revision: 1.130 $
+ *                                                                          *
+ *          Copyright (C) 1992-2002 Free Software Foundation, Inc.          *
+ *                                                                          *
+ * GNAT is free software;  you can  redistribute it  and/or modify it under *
+ * terms of the  GNU General Public License as published  by the Free Soft- *
+ * ware  Foundation;  either version 2,  or (at your option) any later ver- *
+ * sion.  GNAT is distributed in the hope that it will be useful, but WITH- *
+ * OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY *
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
+ * for  more details.  You should have  received  a copy of the GNU General *
+ * Public License  distributed with GNAT;  see file COPYING.  If not, write *
+ * to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, *
+ * MA 02111-1307, USA.                                                      *
+ *                                                                          *
+ * GNAT was originally developed  by the GNAT team at  New York University. *
+ * It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). *
+ *                                                                          *
+ ****************************************************************************/
 
 /* If nonzero, pretend we are allocating at global level.  */
 extern int force_global;
@@ -250,7 +250,7 @@ extern int force_global;
 
 /* Data structures used to represent attributes.  */
 
-enum attr_type {ATTR_MACHINE_ATTRIBUTE, ATTR_LINK_ALIAS, 
+enum attr_type {ATTR_MACHINE_ATTRIBUTE, ATTR_LINK_ALIAS,
 		ATTR_LINK_SECTION, ATTR_WEAK_EXTERNAL};
 
 struct attrib
@@ -305,8 +305,8 @@ extern tree malloc_decl;
 /* Likewise for freeing memory.  */
 extern tree free_decl;
 
-/* Types and decls used by our temporary exception mechanism.  See
-   init_gigi_decls for details.  */
+/* Types and decls used by our exception mechanism.
+   See init_gigi_decls for details.  */
 extern tree jmpbuf_type;
 extern tree jmpbuf_ptr_type;
 extern tree get_jmpbuf_decl;
@@ -315,9 +315,7 @@ extern tree get_excptr_decl;
 extern tree setjmp_decl;
 extern tree longjmp_decl;
 extern tree raise_nodefer_decl;
-extern tree raise_constraint_error_decl;
-extern tree raise_program_error_decl;
-extern tree raise_storage_error_decl;
+extern tree raise_decls[last_reason_code + 1];
 
 /* Routines expected by the gcc back-end. They must have exactly the same
    prototype and names as below.  */
@@ -365,12 +363,12 @@ extern void set_block			PROTO((tree));
    Returns the ..._DECL node. */
 extern tree pushdecl			PROTO((tree));
 
-/* Create the predefined scalar types such as `integer_type_node' needed 
+/* Create the predefined scalar types such as `integer_type_node' needed
    in the gcc back-end and initialize the global binding level.  */
 extern void init_decl_processing	PROTO((void));
 extern void init_gigi_decls		PROTO((tree, tree));
 
-/* Return an integer type with the number of bits of precision given by  
+/* Return an integer type with the number of bits of precision given by
    PRECISION.  UNSIGNEDP is nonzero if the type is unsigned; otherwise
    it is a signed type.  */
 extern tree type_for_size		PROTO((unsigned, int));
@@ -393,7 +391,7 @@ extern tree signed_or_unsigned_type	PROTO((int, tree));
    of an incomplete type.  */
 extern void incomplete_type_error	PROTO((tree, tree));
 
-/* This function is called indirectly from toplev.c to handle incomplete 
+/* This function is called indirectly from toplev.c to handle incomplete
    declarations, i.e. VAR_DECL nodes whose DECL_SIZE is zero.  To be precise,
    compile_file in toplev.c makes an indirect call through the function pointer
    incomplete_decl_finalize_hook which is initialized to this routine in
@@ -413,7 +411,7 @@ extern tree convert			PROTO((tree, tree));
 /* GNAT_ENTITY is a GNAT tree node for a defining identifier.
    GNU_DECL is the GCC tree which is to be associated with
    GNAT_ENTITY. Such gnu tree node is always an ..._DECL node.
-   If NO_CHECK is nonzero, the latter check is suppressed. 
+   If NO_CHECK is nonzero, the latter check is suppressed.
    If GNU_DECL is zero, a previous association is to be reset.  */
 extern void save_gnu_tree		PROTO((Entity_Id, tree, int));
 
@@ -429,7 +427,7 @@ extern int present_gnu_tree	PROTO((Entity_Id));
 extern void init_gnat_to_gnu		PROTO((void));
 
 /* Given a record type (RECORD_TYPE) and a chain of FIELD_DECL
-   nodes (FIELDLIST), finish constructing the record or union type. 
+   nodes (FIELDLIST), finish constructing the record or union type.
    If HAS_REP is nonzero, this record has a rep clause; don't call
    layout_type but merely set the size and alignment ourselves.
    If DEFER_DEBUG is nonzero, do not call the debugging routines
@@ -440,9 +438,9 @@ extern void finish_record_type		PROTO((tree, tree, int, int));
    subprogram. If it is void_type_node, then we are dealing with a procedure,
    otherwise we are dealing with a function. PARAM_DECL_LIST is a list of
    PARM_DECL nodes that are the subprogram arguments.  CICO_LIST is the
-   copy-in/copy-out list to be stored into TYPE_CI_CO_LIST. 
+   copy-in/copy-out list to be stored into TYPE_CI_CO_LIST.
    RETURNS_UNCONSTRAINED is nonzero if the function returns an unconstrained
-   object.  RETURNS_BY_REF is nonzero if the function returns by reference. 
+   object.  RETURNS_BY_REF is nonzero if the function returns by reference.
    RETURNS_WITH_DSP is nonzero if the function is to return with a
    depressed stack pointer.  */
 extern tree create_subprog_type		PROTO((tree, tree, tree, int, int,
@@ -457,7 +455,7 @@ extern tree copy_type			PROTO((tree));
 extern tree create_index_type		PROTO((tree, tree, tree));
 
 /* Return a TYPE_DECL node. TYPE_NAME gives the name of the type (a character
-   string) and TYPE is a ..._TYPE node giving its data type. 
+   string) and TYPE is a ..._TYPE node giving its data type.
    ARTIFICIAL_P is nonzero if this is a declaration that was generated
    by the compiler.  DEBUG_INFO_P is nonzero if we need to write debugging
    information about this type.  */
@@ -473,7 +471,7 @@ extern tree create_type_decl		PROTO((tree, tree, struct attrib *,
 
    PUBLIC_FLAG is nonzero if this definition is to be made visible outside of
    the current compilation unit. This flag should be set when processing the
-   variable definitions in a package specification.  EXTERN_FLAG is nonzero 
+   variable definitions in a package specification.  EXTERN_FLAG is nonzero
    when processing an external variable declaration (as opposed to a
    definition: no storage is to be allocated for the variable here).
    STATIC_FLAG is only relevant when not at top level.  In that case
@@ -576,8 +574,10 @@ extern void update_pointer_to	PROTO((tree, tree));
 extern tree max_size		PROTO((tree, int));
 
 /* Remove all conversions that are done in EXP.  This includes converting
-   from a padded type or converting to a left-justified modular type.  */
-extern tree remove_conversions	PROTO((tree));
+   from a padded type or some conversions to a left-justified modular type.
+   If FOR_ADDRESS is nonzero, we are going to be taking the address of the
+   result, so we need to only remove some LJM conversions.  */
+extern tree remove_conversions	PROTO((tree, int));
 
 /* If EXP's type is an UNCONSTRAINED_ARRAY_TYPE, return an expression that
    refers to the underlying array.  If its type has TYPE_CONTAINS_TEMPLATE_P,
