@@ -1,30 +1,30 @@
-/****************************************************************************/
-/*                                                                          */
-/*                         GNAT COMPILER COMPONENTS                         */
-/*                                                                          */
-/*                             A - T R A N S 4                              */
-/*                                                                          */
-/*                          C Implementation File                           */
-/*                                                                          */
-/*                            $Revision: 1.181 $                            */
-/*                                                                          */
-/*          Copyright (C) 1992-2000, Free Software Foundation, Inc.         */
-/*                                                                          */
-/* GNAT is free software;  you can  redistribute it  and/or modify it under */
-/* terms of the  GNU General Public License as published  by the Free Soft- */
-/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
-/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
-/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
-/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
-/* for  more details.  You should have  received  a copy of the GNU General */
-/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
-/* MA 02111-1307, USA.                                                      */
-/*                                                                          */
-/* GNAT was originally developed  by the GNAT team at  New York University. */
-/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
-/*                                                                          */
-/****************************************************************************/
+/****************************************************************************
+ *                                                                          *
+ *                         GNAT COMPILER COMPONENTS                         *
+ *                                                                          *
+ *                             A - T R A N S 4                              *
+ *                                                                          *
+ *                          C Implementation File                           *
+ *                                                                          *
+ *                            $Revision: 1.193 $
+ *                                                                          *
+ *          Copyright (C) 1992-2002, Free Software Foundation, Inc.         *
+ *                                                                          *
+ * GNAT is free software;  you can  redistribute it  and/or modify it under *
+ * terms of the  GNU General Public License as published  by the Free Soft- *
+ * ware  Foundation;  either version 2,  or (at your option) any later ver- *
+ * sion.  GNAT is distributed in the hope that it will be useful, but WITH- *
+ * OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY *
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
+ * for  more details.  You should have  received  a copy of the GNU General *
+ * Public License  distributed with GNAT;  see file COPYING.  If not, write *
+ * to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, *
+ * MA 02111-1307, USA.                                                      *
+ *                                                                          *
+ * GNAT was originally developed  by the GNAT team at  New York University. *
+ * It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). *
+ *                                                                          *
+ ****************************************************************************/
 
 #include "config.h"
 #include "tree.h"
@@ -291,7 +291,7 @@ compare_arrays (result_type, a1, a2)
       tree comparison, this_a1_is_null, this_a2_is_null;
 
       /* If the length of the first array is a constant, swap our operands
-	 unless the length of the second array is the constant zero.  
+	 unless the length of the second array is the constant zero.
 	 Note that we have set the `length' values to the length - 1.  */
       if (TREE_CODE (length1) == INTEGER_CST
 	  && ! integer_zerop (fold (build (PLUS_EXPR, bt, length2,
@@ -340,7 +340,7 @@ compare_arrays (result_type, a1, a2)
 	  nbt = get_base_type (TREE_TYPE (ub1));
 
 	  comparison
-	    = build_binary_op (EQ_EXPR, result_type, 
+	    = build_binary_op (EQ_EXPR, result_type,
 			       build_binary_op (MINUS_EXPR, nbt, ub1, lb1),
 			       build_binary_op (MINUS_EXPR, nbt, ub2, lb2));
 
@@ -396,9 +396,8 @@ compare_arrays (result_type, a1, a2)
       if (type != 0)
 	a1 = convert (type, a1), a2 = convert (type, a2);
 
-
       result = build_binary_op (TRUTH_ANDIF_EXPR, result_type, result,
-				build (EQ_EXPR, result_type, a1, a2));
+				fold (build (EQ_EXPR, result_type, a1, a2)));
 
     }
 
@@ -548,10 +547,11 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 
   /* If one (but not both, unless they have the same object) operands are a
      WITH_RECORD_EXPR, do the operation and then surround it with the
-     WITH_RECORD_EXPR.  Don't do this for assignment or for an ARRAY_REF
-     because we need to keep track of the WITH_RECORD_EXPRs on both
-     operands very carefully.  */
+     WITH_RECORD_EXPR.  Don't do this for assignment, for an ARRAY_REF, or
+     for an ARRAY_RANGE_REF because we need to keep track of the
+     WITH_RECORD_EXPRs on both operands very carefully.  */
   if (op_code != MODIFY_EXPR && op_code != ARRAY_REF
+      && op_code != ARRAY_RANGE_REF
       && TREE_CODE (left_operand) == WITH_RECORD_EXPR
       && (TREE_CODE (right_operand) != WITH_RECORD_EXPR
 	  || operand_equal_p (TREE_OPERAND (left_operand, 1),
@@ -568,6 +568,7 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 		    TREE_OPERAND (left_operand, 1));
     }
   else if (op_code != MODIFY_EXPR && op_code != ARRAY_REF
+	   && op_code != ARRAY_RANGE_REF
 	   && TREE_CODE (left_operand) != WITH_RECORD_EXPR
 	   && TREE_CODE (right_operand) == WITH_RECORD_EXPR)
     {
@@ -676,14 +677,15 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	  && TYPE_ALIGN_OK_P (right_type))
 	operation_type = right_type;
 
-      /* After we strip off any COMPONENT_REF or ARRAY_REF from the lhs, we
-	 must have either an INDIRECT_REF or a decl. Allow
+      /* After we strip off any COMPONENT_REF or ARRAY_REF, or ARRAY_RANGE_REF,
+	 from the lhs, we must have either an INDIRECT_REF or a decl. Allow
 	 UNCHECKED_CONVERT_EXPRs, but set TREE_ADDRESSABLE to show they are
 	 in an LHS.  Finally, allow NOP_EXPR if both types are the same tree
 	 code and mode because we know these will be nops.  */
       for (result = left_operand;
 	   TREE_CODE (result) == COMPONENT_REF
 	   || TREE_CODE (result) == ARRAY_REF
+	   || TREE_CODE (result) == ARRAY_RANGE_REF
 	   || TREE_CODE (result) == REALPART_EXPR
 	   || TREE_CODE (result) == IMAGPART_EXPR
 	   || TREE_CODE (result) == WITH_RECORD_EXPR
@@ -744,6 +746,10 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
       if (operation_type == 0)
 	operation_type = TREE_TYPE (left_type);
 
+      /* ... fall through ... */
+
+    case ARRAY_RANGE_REF:
+
       /* First convert the right operand to its base type.  This will
 	 prevent unneed signedness conversions when sizetype is wider than
 	 integer.  */
@@ -751,7 +757,8 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
       right_operand = convert (TYPE_DOMAIN (left_type), right_operand);
 
       if (! TREE_CONSTANT (right_operand)
-	  || ! TREE_CONSTANT (TYPE_MIN_VALUE (right_type)))
+	  || ! TREE_CONSTANT (TYPE_MIN_VALUE (right_type))
+	  || op_code == ARRAY_RANGE_REF)
 	mark_addressable (left_operand);
 
       /* If the array is an UNCHECKED_CONVERT_EXPR from and to BLKmode
@@ -882,7 +889,7 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	    gigi_abort (505);
 	}
 
-      /* If we are comparing a fat pointer against zero, we need to 
+      /* If we are comparing a fat pointer against zero, we need to
 	 just compare the data pointer.  */
       else if (TYPE_FAT_POINTER_P (left_base_type)
 	       && TREE_CODE (right_operand) == CONSTRUCTOR
@@ -1005,10 +1012,11 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 
   TREE_SIDE_EFFECTS (result) |= has_side_effects;
   TREE_CONSTANT (result)
-    = (TREE_CONSTANT (left_operand) & TREE_CONSTANT (right_operand)
-       && op_code != ARRAY_REF);
+    |= (TREE_CONSTANT (left_operand) & TREE_CONSTANT (right_operand)
+	&& op_code != ARRAY_REF && op_code != ARRAY_RANGE_REF);
 
-  if (op_code == ARRAY_REF && TYPE_VOLATILE (operation_type))
+  if ((op_code == ARRAY_REF || op_code == ARRAY_RANGE_REF)
+      && TYPE_VOLATILE (operation_type))
     TREE_THIS_VOLATILE (result) = 1;
 
   /* If we are working with modular types, perform the MOD operation
@@ -1108,66 +1116,56 @@ build_unary_op (op_code, result_type, operand)
 	  break;
 
 	case ARRAY_REF:
+	case ARRAY_RANGE_REF:
 	case COMPONENT_REF:
 	case BIT_FIELD_REF:
-	  {
-	    tree op_type = TREE_TYPE (TREE_OPERAND (operand, 0));
-	    int bitsize, bitpos;
-	    tree offset, inner;
-	    enum machine_mode mode;
-	    int unsignedp, volatilep, alignment;
+	    /* If this is for 'Address, find the address of the prefix and
+	       add the offset to the field.  Otherwise, do this the normal
+	       way.  */
+	  if (op_code == ATTR_ADDR_EXPR)
+	    {
+	      tree op_type = TREE_TYPE (TREE_OPERAND (operand, 0));
+	      int bitsize, bitpos;
+	      tree offset, inner;
+	      enum machine_mode mode;
+	      int unsignedp, volatilep, alignment;
 
-	    inner = get_inner_reference (operand, &bitsize, &bitpos, &offset,
-					 &mode, &unsignedp, &volatilep,
-					 &alignment);
+	      inner = get_inner_reference (operand, &bitsize, &bitpos, &offset,
+					   &mode, &unsignedp, &volatilep,
+					   &alignment);
 
-	    /* If this is a bitfield and not part of the 'Address
-	       attribute, let this be done the normal way, which will make a 
-	       temporary containing the value and take the address of that.
-	       Likewise if we didn't find anything different for the inner
-	       operand or if OPERAND extracts direction from a padding type
-	       whose field has a self-referential size (to avoid infinite
-	       recursion below) or if the size of the type or the offset
-	       are self-referential since we need an object in that case.  */
-	    if ((mode == VOIDmode && op_code != ATTR_ADDR_EXPR)
-		|| inner == operand
-		|| (TREE_CODE (op_type) == RECORD_TYPE
-		    && TYPE_IS_PADDING_P (op_type)
-		    && (contains_placeholder_p
-			(TYPE_SIZE (TREE_TYPE (TYPE_FIELDS (op_type))))))
-		|| contains_placeholder_p (TYPE_SIZE (op_type))
-		|| (offset != 0 && contains_placeholder_p (offset)))
-	      goto common;
+	      /* If INNER is a padding type whose field has a self-referential
+		 size, convert to that inner type.  We know the offset is zero
+		 and we need to have that type visible.  */
+	      if (TREE_CODE (TREE_TYPE (inner)) == RECORD_TYPE
+		  && TYPE_IS_PADDING_P (TREE_TYPE (inner))
+		  && (contains_placeholder_p
+		      (TYPE_SIZE (TREE_TYPE (TYPE_FIELDS
+					     (TREE_TYPE (inner)))))))
+		inner = convert (TREE_TYPE (TYPE_FIELDS (TREE_TYPE (inner))),
+				 inner);
 
-	    /* If INNER is a padding type whose field has a self-referential
-	       size, convert to that inner type.  We know the offset is zero
-	       and we need to have that type visible.  */
-	    if (TREE_CODE (TREE_TYPE (inner)) == RECORD_TYPE
-		&& TYPE_IS_PADDING_P (TREE_TYPE (inner))
-		&& (contains_placeholder_p
-		    (TYPE_SIZE (TREE_TYPE (TYPE_FIELDS (TREE_TYPE (inner)))))))
-	      inner = convert (TREE_TYPE (TYPE_FIELDS (TREE_TYPE (inner))),
-			       inner);
+	      /* Compute the offset as a byte offset from INNER.  */
+	      if (offset == 0)
+		offset = size_zero_node;
 
-	    /* Compute the offset as a byte offset from INNER.  */
-	    if (offset == 0)
-	      offset = size_zero_node;
+	      offset = size_binop (PLUS_EXPR, offset,
+				   size_int (bitpos / BITS_PER_UNIT));
 
-	    offset = size_binop (PLUS_EXPR, offset,
-				 size_int (bitpos / BITS_PER_UNIT));
+	      /* Take the address of INNER, convert the offset to void *, and
+		 add then.  It will later be converted to the desired result
+		 type, if any.  */
+	      inner = build_unary_op (ADDR_EXPR, NULL_TREE, inner);
+	      inner = convert (ptr_void_type_node, inner);
+	      offset = convert (ptr_void_type_node, offset);
+	      result = build_binary_op (PLUS_EXPR, ptr_void_type_node,
+					inner, offset);
+	      result = convert (build_pointer_type (TREE_TYPE (operand)),
+				result);
+	      break;
+	    }
 
-	    /* Take the address of INNER, convert the offset to void *, and
-	       add then.  It will later be converted to the desired result
-	       type, if any.  */
-	    inner = build_unary_op (ADDR_EXPR, NULL_TREE, inner);
-	    inner = convert (ptr_void_type_node, inner);
-	    offset = convert (ptr_void_type_node, offset);
-	    result = build_binary_op (PLUS_EXPR, ptr_void_type_node,
-				      inner, offset);
-	    result = convert (build_pointer_type (TREE_TYPE (operand)),
-			      result);
-	  }
-	  break;
+	  goto common;
 
 	case CONSTRUCTOR:
 	  /* If this is just a constructor for a padded record, we can
@@ -1250,7 +1248,7 @@ build_unary_op (op_code, result_type, operand)
 	    = TREE_READONLY (TREE_TYPE (type));
 	}
 
-      side_effects = flag_volatile 
+      side_effects = flag_volatile
 	|| (! TYPE_FAT_POINTER_P (type) && TYPE_VOLATILE (TREE_TYPE (type)));
       break;
 
@@ -1447,9 +1445,10 @@ build_call_0_expr (fundecl)
    number and file name, if requested.  */
 
 tree
-build_call_raise (fndecl)
-     tree fndecl;
+build_call_raise (msg)
+     int msg;
 {
+  tree fndecl = raise_decls [msg];
   char *str = discard_file_names ? "" : ref_filename;
   int len = strlen (str) + 1;
   tree filename = build_string (len, str);
@@ -1460,9 +1459,8 @@ build_call_raise (fndecl)
 
   return
     build_call_2_expr (fndecl,
-		       build1 (ADDR_EXPR, build_pointer_type (char_type_node),
-			       filename),
-		       build_int_2 (lineno, 0));
+      build1 (ADDR_EXPR, build_pointer_type (char_type_node), filename),
+      build_int_2 (lineno, 0));
 }
 
 /* Return a CONSTRUCTOR of TYPE whose list is LIST.  */
@@ -1585,7 +1583,7 @@ build_simple_component_ref (record_variable, component, field)
 	  if (DECL_INTERNAL_P (new_field))
 	    {
 	      tree field_ref
-		= build_simple_component_ref (record_variable, 
+		= build_simple_component_ref (record_variable,
 					      NULL_TREE, new_field);
 	      ref = build_simple_component_ref (field_ref, NULL_TREE, field);
 
@@ -1601,11 +1599,15 @@ build_simple_component_ref (record_variable, component, field)
 
   /* If the record variable is an UNCHECKED_CONVERT_EXPR from and to BLKmode
      types, convert it to a normal conversion since GCC can deal with any
-     mis-alignment as part of the handling of compponent references.  */
+     mis-alignment as part of the handling of compponent references.  If we
+
+     need to tell GCC that this is not misaligned, do so.  */
   if (TREE_CODE (record_variable) == UNCHECKED_CONVERT_EXPR
       && TYPE_MODE (TREE_TYPE (record_variable)) == BLKmode
       && TYPE_MODE (TREE_TYPE (TREE_OPERAND (record_variable, 0))) == BLKmode)
-    record_variable = build1 (CONVERT_EXPR, TREE_TYPE (record_variable),
+    record_variable = build1 (TYPE_ALIGN_OK_P (TREE_TYPE (record_variable))
+			      ? CONVERT_EXPR : NOP_EXPR,
+			      TREE_TYPE (record_variable),
 			      TREE_OPERAND (record_variable, 0));
 
   /* It would be nice to call "fold" here, but that can lose a type
@@ -1641,7 +1643,7 @@ build_component_ref (record_variable, component, field)
 
   else if (field != 0)
     return build1 (NULL_EXPR, TREE_TYPE (field),
-		   build_call_raise (raise_constraint_error_decl));
+		   build_call_raise (CE_Discriminant_Check_Failed));
   else
     gigi_abort (512);
 }
@@ -1674,7 +1676,7 @@ build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
 
   if (Present (gnat_proc))
 
-    /* The storage pools are obviously always tagged types, but the 
+    /* The storage pools are obviously always tagged types, but the
        secondary stack uses the same mechanism and is not tagged */
 
     if (Is_Tagged_Type (Etype (gnat_pool)))
@@ -1700,13 +1702,13 @@ build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
 	if (gnu_obj)
 	  gnu_args = chainon (gnu_args, build_tree_list (NULL_TREE, gnu_obj));
 
-	gnu_args = 
+	gnu_args =
 	  chainon (gnu_args,
 		   build_tree_list (NULL_TREE,
 				    convert (gnu_size_type, gnu_size)));
-	gnu_args = 
+	gnu_args =
 	  chainon (gnu_args,
-		   build_tree_list (NULL_TREE, 
+		   build_tree_list (NULL_TREE,
 				    convert (gnu_size_type, gnu_align)));
 
 	gnu_call = build (CALL_EXPR, TREE_TYPE (TREE_TYPE (gnu_proc)),
@@ -1720,7 +1722,7 @@ build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
     else
       {
 	/* The size is the second parameter */
-	Entity_Id gnat_size_type 
+	Entity_Id gnat_size_type
 	  = Etype (Next_Formal (First_Formal (gnat_proc)));
 	tree gnu_size_type = gnat_to_gnu_type (gnat_size_type);
 	tree gnu_proc = gnat_to_gnu (gnat_proc);
@@ -1736,7 +1738,7 @@ build_call_alloc_dealloc (gnu_obj, gnu_size, align, gnat_proc, gnat_pool)
 	if (gnu_obj)
 	  gnu_args = chainon (gnu_args, build_tree_list (NULL_TREE, gnu_obj));
 
-	gnu_args = 
+	gnu_args =
 	  chainon (gnu_args,
 		   build_tree_list (NULL_TREE,
 				    convert (gnu_size_type, gnu_size)));
@@ -1802,6 +1804,11 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
 	  && contains_placeholder_p (size))
 	size = build (WITH_RECORD_EXPR, sizetype, size, init);
 
+      /* If the size overflows, pass -1 so the allocator will raise
+	 storage error.  */
+      if (TREE_CODE (size) == INTEGER_CST && TREE_OVERFLOW (size))
+	size = convert (sizetype, build_int_2 (-BITS_PER_UNIT, ~0));
+
       storage = build_call_alloc_dealloc (NULL_TREE, size,
 					  TYPE_ALIGN (storage_type),
 					  gnat_proc, gnat_pool);
@@ -1920,7 +1927,7 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
   return convert (result_type, result);
 }
 
-/* Fill in a VMS descriptor for EXPR and return a constructor for it. 
+/* Fill in a VMS descriptor for EXPR and return a constructor for it.
    GNAT_FORMAL is how we find the descriptor record.  */
 
 tree
@@ -1963,6 +1970,7 @@ mark_addressable (expr_node)
       case ADDR_EXPR:
       case COMPONENT_REF:
       case ARRAY_REF:
+      case ARRAY_RANGE_REF:
       case REALPART_EXPR:
       case IMAGPART_EXPR:
       case NOP_EXPR:
