@@ -1,30 +1,30 @@
-/****************************************************************************/
-/*                                                                          */
-/*                         GNAT COMPILER COMPONENTS                         */
-/*                                                                          */
-/*                             A - T R A N S 4                              */
-/*                                                                          */
-/*                              C Header File                               */
-/*                                                                          */
-/*                            $Revision: 1.35 $                             */
-/*                                                                          */
-/*          Copyright (C) 1992-1999 Free Software Foundation, Inc.          */
-/*                                                                          */
-/* GNAT is free software;  you can  redistribute it  and/or modify it under */
-/* terms of the  GNU General Public License as published  by the Free Soft- */
-/* ware  Foundation;  either version 2,  or (at your option) any later ver- */
-/* sion.  GNAT is distributed in the hope that it will be useful, but WITH- */
-/* OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY */
-/* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
-/* for  more details.  You should have  received  a copy of the GNU General */
-/* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
-/* MA 02111-1307, USA.                                                      */
-/*                                                                          */
-/* GNAT was originally developed  by the GNAT team at  New York University. */
-/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
-/*                                                                          */
-/****************************************************************************/
+/****************************************************************************
+ *                                                                          *
+ *                         GNAT COMPILER COMPONENTS                         *
+ *                                                                          *
+ *                             A - T R A N S 4                              *
+ *                                                                          *
+ *                              C Header File                               *
+ *                                                                          *
+ *                            $Revision: 1.37 $
+ *                                                                          *
+ *          Copyright (C) 1992-2002 Free Software Foundation, Inc.          *
+ *                                                                          *
+ * GNAT is free software;  you can  redistribute it  and/or modify it under *
+ * terms of the  GNU General Public License as published  by the Free Soft- *
+ * ware  Foundation;  either version 2,  or (at your option) any later ver- *
+ * sion.  GNAT is distributed in the hope that it will be useful, but WITH- *
+ * OUT ANY WARRANTY;  without even the  implied warranty of MERCHANTABILITY *
+ * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License *
+ * for  more details.  You should have  received  a copy of the GNU General *
+ * Public License  distributed with GNAT;  see file COPYING.  If not, write *
+ * to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, *
+ * MA 02111-1307, USA.                                                      *
+ *                                                                          *
+ * GNAT was originally developed  by the GNAT team at  New York University. *
+ * It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). *
+ *                                                                          *
+ ****************************************************************************/
 
 /* Prepare expr to be an argument of a TRUTH_NOT_EXPR or other logical
    operation.
@@ -68,16 +68,17 @@ extern tree build_cond_expr	PROTO((tree, tree, tree, tree));
    the CALL_EXPR.  */
 extern tree build_call_1_expr	PROTO((tree, tree));
 
-/* Build a CALL_EXPR to call FUNDECL with two argument, ARG1 & ARG2.  Return
-   the CALL_EXPR.  */
+/* Build a CALL_EXPR to call FUNDECL with two arguments, ARG1 & ARG2.
+   Return the CALL_EXPR.  */
 extern tree build_call_2_expr	PROTO((tree, tree, tree));
 
 /* Likewise to call FUNDECL with no arguments.  */
 extern tree build_call_0_expr	PROTO((tree));
 
 /* Call a function FCN that raises an exception and pass the line
-   number and file name, if requested.  */
-extern tree build_call_raise	PROTO((tree));
+   number and file name, if requested. Second parameter is the Reason
+   code, as defined in a-types.h */
+extern tree build_call_raise	PROTO((int));
 
 /* Return a CONSTRUCTOR of TYPE whose list is LIST.  */
 extern tree build_constructor	PROTO((tree, tree));
@@ -99,13 +100,13 @@ extern tree build_call_alloc_dealloc PROTO((tree, tree, int, Entity_Id,
 
 /* Build a GCC tree to correspond to allocating an object of TYPE whose
    initial value if INIT, if INIT is nonzero.  Convert the expression to
-   RESULT_TYPE, which must be some type of pointer.  Return the tree. 
+   RESULT_TYPE, which must be some type of pointer.  Return the tree.
    GNAT_PROC and GNAT_POOL optionally give the procedure to call and
    the storage pool to use.  */
 extern tree build_allocator	PROTO((tree, tree, tree, Entity_Id,
 				       Entity_Id));
 
-/* Fill in a VMS descriptor for EXPR and return a constructor for it. 
+/* Fill in a VMS descriptor for EXPR and return a constructor for it.
    GNAT_FORMAL is how we find the descriptor record.  */
 
 extern tree fill_vms_descriptor PROTO((tree, Entity_Id));
