@@ -6,7 +6,7 @@
 #-                                                                          --
 #-                             G N A T H T M L                              --
 #-                                                                          --
-#-                            $Revision: 1.29 $                             --
+#-                            $Revision: 1.34 $                             --
 #-                                                                          --
 #-          Copyright (C) 1998 Free Software Foundation, Inc.               --
 #-                                                                          --
@@ -322,6 +322,8 @@ sub output_line_number
 ########
 ##  Converts a character into the corresponding Ada type
 ##  This is based on the ali format (see lib-xref.adb) in the GNAT sources
+##  Note: 'f' or 'K' should be returned in case a link from the body to the
+##        spec needs to be generated.
 ##  Params : - the character to convert
 ########
 sub to_type
@@ -330,22 +332,26 @@ sub to_type
   $char =~ tr/a-z/A-Z/;
   
   return 'array'                              if ($char eq 'A');
+  return 'boolean'                            if ($char eq 'B');
   return 'class'                              if ($char eq 'C');
   return 'decimal'                            if ($char eq 'D');
   return 'enumeration'                        if ($char eq 'E');
   return 'floating point'                     if ($char eq 'F');
-  return 'generic function/package/procedure' if ($char eq 'G');
-  return 'integer'                            if ($char eq 'I');
+  return 'signed integer'                     if ($char eq 'I');
+  # return 'generic package'                    if ($char eq 'K');
   return 'block'                              if ($char eq 'L');
   return 'modular integer'                    if ($char eq 'M');
   return 'enumeration litteral'               if ($char eq 'N');
   return 'ordinary fixed point'               if ($char eq 'O');
   return 'access'                             if ($char eq 'P');
+  return 'label'                              if ($char eq 'Q');  
   return 'record'                             if ($char eq 'R');
   return 'string'                             if ($char eq 'S');
   return 'task'                               if ($char eq 'T');
   return 'f'                                  if ($char eq 'U');
+  return 'f'                                  if ($char eq 'V');
   return 'exception'                          if ($char eq 'X');
+  return 'entry'                              if ($char eq 'Y');
   return "$char";
 }
 
@@ -374,33 +380,20 @@ sub get_real_file_name
   local ($filename) = shift;
   local ($path) = $filename;
   
-  if ($filename =~ /.ali$/)
+  foreach (@src_dir)
   {
-    foreach (@obj_dir)
-    {
       if ( -r "$_$filename")
       {
-	$path = "$_$filename";
-	last;
+	  $path = "$_$filename";
+	  last;
       }
-    }
-  }
-  else
-  {
-    foreach (@src_dir)
-    {
-      if ( -r "$_$filename")
-      {
-	$path = "$_$filename";
-	last;
-      }
-    }
   }
   
   $path =~ s/^\.\///;
   return $path if (substr ($path, 0, 1) ne '/');
-  
-  ## Search the relative path
+
+  ## We want to return relative paths only, so that the name of the HTML files
+  ## can easily be generated
   local ($pwd) = `pwd`;
   chop ($pwd);
   local (@pwd) = split (/\//, $pwd);
@@ -485,7 +478,14 @@ sub create_new_reference
     
     ($refline, $type, $refcol) = /(\d+)(.)(\d+)/;
     $refline += $offset;
-    if ($type eq "b" && $symbols {$currentref} eq 'f')
+
+
+    ## If we have a body, then we only generate the cross-reference from
+    ## the spec to the body if we have a subprogram (or a package)
+    
+    
+    if ($type eq "b")
+#	&& ($symbols {$currentref} eq 'f' || $symbols {$currentref} eq 'K'))
     {
 	local ($cref_file, $cref) = ($currentref =~ /([^\#]+).htm\#(.+)/);
 
@@ -496,8 +496,10 @@ sub create_new_reference
 
     ## Do not generate cross-references for "e" and "t", since these point to the
     ## semicolon that terminates the block -- irrelevant for gnathtml
+    ## "p" is also removed, since it is used for primitive subprograms
+    ## "i" is removed since it is used for implicit references
     
-    elsif ($type ne "e" && $type ne "t") 
+    elsif ($type ne "e" && $type ne "t" && $type ne "p" && $type ne "i") 
     {
 	$symbols_used {"$lastfile#$refline\_$refcol"} = $currentref;
     }
@@ -514,31 +516,45 @@ sub parse_ali
   local ($currentref);
   local ($lastfile);
 
+  # A    file | line type column      reference
+  local ($reference) = "(?:(?:\\d+\\|)?\\d+.\\d+|\\w+)";
+
+  # The following variable is used to represent the possible xref information
+  # output by GNAT when -gnatdM is used. It includes renaming references, and
+  # references to the parent type.
+
+  local ($typeref) = "(?:=$reference|<$reference>|\\{$reference\\}|\\($reference\\))?";
+
+  # The beginning of an entity declaration line in the ALI file
+  local ($decl_line) = "^(\\d+)(.)(\\d+)[ *]([\\w\\d.-]+|\"..?\")$typeref\\s+(\\S.*)?\$";
+ 
   # Contains entries of the form  [ filename source_reference_offset]
   # Offset needs to be added to the lines read in the cross-references, and are
   # used when the source comes from a gnatchop-ed file. See lib-write.ads, lines
   # with ^D in the ALI file.
   local (@reffiles) = ();
-  
-  
+
   open (ALI, &find_file ($filename)) || do {
     print "no ", &find_file ($filename), " file...\n";
     return;
   };
   local (@ali) = <ALI>;
   close (ALI);
-  
+
   undef %symbols;
   undef %symbols_used;
-  
+
   foreach (@ali)
   {
-    if (/^D\s+([\w\d.-]+)\s+\S+ \S+( (\d+):(.*))?/)
+    ## The format of D lines is
+    ## D source-name time-stamp checksum [subunit-name] line:file-name
+
+    if (/^D\s+([\w\d.-]+)\s+\S+ \S+(\s+\D[^: ]+)?( (\d+):(.*))?/)
     {
 	# The offset will be added to each cross-reference line. If it is
 	# greater than 1, this means that we have a pragma Source_Reference,
 	# and this must not be counted in the xref information.
-	my ($file, $offset) = ($1, (defined $3) ? 2 - $3 : 0);
+	my ($file, $offset) = ($1, (defined $4) ? 2 - $4 : 0);
 
 	if ($dependencies)
 	{
@@ -551,8 +567,8 @@ sub parse_ali
     {
 	$currentfile = $lastfile = $1 - 1;
     }
-    
-    elsif (/^(\d+)(.)(\d+)[ *]([\w\d-.]|".")+\s+(.*)/ && defined $currentfile)
+
+    elsif (defined $currentfile && /$decl_line/)
     {
       my ($line) = $1 + $reffiles[$currentfile][1];
       next if (! $standard_library
@@ -593,6 +609,67 @@ sub parse_ali
   }
 }
 
+#########
+##  Return the name of the ALI file to use for a given source
+##  Params:  - Name of the source file
+##  return:  Name and location of the ALI file
+#########
+
+sub ali_file_name {
+    local ($source) = shift;
+    local ($alifilename, $unitname);
+    local ($in_separate) = 0;
+
+    $source =~ s/\.ad[sb]$//;
+    $alifilename = $source;
+    $unitname = $alifilename;
+    $unitname =~ s/-/./g;
+
+    ## There are two reasons why we might not find the ALI file: either the
+    ## user did not generate them at all, or we are working on a separate unit.
+    ## Thus, we search in the parent's ALI file.
+
+    while ($alifilename ne "") {
+
+      ## Search in the object path
+      foreach (@obj_dir) {
+
+	## Check if the ALI file does apply to the source file
+	## We check the ^D lines, which have the following format:
+	## D source-name time-stamp checksum [subunit-name] line:file-name
+
+	if (-r "$_$alifilename.ali") {
+	  if ($in_separate) {
+	    open (FILE, "$_$alifilename.ali");
+
+	    if (grep (/^D \S+\s+\S+\s+\S+ $unitname/, <FILE>)) {
+	      close FILE;
+	      return "$_$alifilename.ali";
+
+	    } else {
+	      ## If the ALI file doesn't apply to the source file, we can
+	      ## return now, since there won't be a parent ALI file above
+	      ## anyway
+	      close FILE;
+	      return "$source.ali";
+	    }
+	  } else {
+	    return "$_$alifilename.ali";
+	  }
+	}
+      }
+
+      ## Get the parent's ALI file name
+
+      if (! ($alifilename =~ s/-[^-]+$//)) {
+	$alifilename = "";
+      }
+      $in_separate = 1;
+    }
+
+    return "$source.ali";
+}
+
 #########
 ##  This function outputs the html version of the file FILE
 ##  The output is send to FILE.htm.
@@ -604,8 +681,7 @@ sub output_file
   local ($lineno)   = 1;
   local ($column);
   
-  local ($alifilename) = $filename_param;
-  $alifilename =~ s/\.ad[sb]$/.ali/;
+  local ($alifilename) = &ali_file_name ($filename_param);
   
   $filename = &get_real_file_name ($filename_param);
   
@@ -619,7 +695,7 @@ sub output_file
 
   ## Parse the .ali file to find the cross-references
   print "converting ", $filename, "\n";
-  &parse_ali (&get_real_file_name ($alifilename));
+  &parse_ali ($alifilename);
 
   ## Create and initialize the html file
   open (OUTPUT, ">$output_dir/" . &http_string ($filename) . ".htm")
@@ -689,6 +765,7 @@ sub output_file
 
 		  local ($lowercase) = $word;
 		  $lowercase =~ tr/A-Z/a-z/;
+
 		  if ($keywords{$lowercase})
 		  {
 		      print OUTPUT &output_keyword ($word);
