@@ -1589,6 +1589,8 @@ print_octal (value, digits)
 
   for (i = digits - 1; i >= 0; i--)
     fprintf (asmfile, "%01o", ((value >> (3 * i)) & 7));
+
+  CHARS (digits);
 }
 
 /* Output the name of type TYPE, with no punctuation.
@@ -1733,7 +1735,12 @@ dbxout_symbol (decl, local)
 		&& !TREE_ASM_WRITTEN (TYPE_NAME (type))
 		/* Distinguish the implicit typedefs of C++
 		   from explicit ones that might be found in C.  */
-                && DECL_ARTIFICIAL (decl))
+                && DECL_ARTIFICIAL (decl)
+		/* Do not generate a tag for records of variable size,
+		   since this type can not be properly described in the
+		   DBX format, and it confuses some tools such as objdump */
+		&& !(TREE_CODE(type) == RECORD_TYPE &&
+		     TREE_CODE (TYPE_SIZE (type)) != INTEGER_CST))
 	      {
 		tree name = TYPE_NAME (type);
 		if (TREE_CODE (name) == TYPE_DECL)
@@ -2375,9 +2382,22 @@ dbxout_parms (parms)
 	        current_sym_value = 0;
 	      }
 	    else
+	      {
 		current_sym_value
 		  = INTVAL (XEXP (XEXP (DECL_RTL (parms), 0), 1));
 
+		/* Make a big endian correction if the mode of the type of the
+		   parameter is not the same as the mode of the rtl.  */
+		if (BYTES_BIG_ENDIAN
+		    && TYPE_MODE (TREE_TYPE (parms)) != GET_MODE (DECL_RTL (parms))
+		    && GET_MODE_SIZE (TYPE_MODE (TREE_TYPE (parms))) < UNITS_PER_WORD)
+		  {
+		    current_sym_value += 
+		      GET_MODE_SIZE (GET_MODE (DECL_RTL (parms)))
+		      - GET_MODE_SIZE (TYPE_MODE (TREE_TYPE (parms)));
+		  }
+	      }
+		
 	    current_sym_addr = 0;
 
 	    FORCE_TEXT;
@@ -2402,6 +2422,7 @@ dbxout_parms (parms)
 				     XEXP (DECL_RTL (parms), 0));
 	    dbxout_type (TREE_TYPE (parms), 0, 0);
 	    dbxout_finish_symbol (parms);
+
 	    if (aux_sym_value != 0)
 	      {
 		/* Generate an entry for the stack location */
@@ -2412,8 +2433,8 @@ dbxout_parms (parms)
 	        current_sym_code = N_LSYM;
 	        dbxout_type (build_reference_type (TREE_TYPE (parms)), 0, 0);
 	        dbxout_finish_symbol (parms);
-	      }
-	  }
+	      }	 
+          }
       }
 }
 
