@@ -1475,7 +1475,7 @@ __bb_exit_func (void)
 	  /* If the file exists, and the number of counts in it is the same,
 	     then merge them in.  */
 	     
-	  if ((da_file = fopen (ptr->filename, "r")) != 0)
+	  if ((da_file = fopen (ptr->filename, "rb")) != 0)
 	    {
 	      long n_counts = 0;
 	      unsigned char tmp;
@@ -1515,7 +1515,7 @@ __bb_exit_func (void)
 		fprintf (stderr, "arc profiling: Error closing output file %s.\n",
 			 ptr->filename);
 	    }
-	  if ((da_file = fopen (ptr->filename, "w")) == 0)
+	  if ((da_file = fopen (ptr->filename, "wb")) == 0)
 	    {
 	      fprintf (stderr, "arc profiling: Can't open output file %s.\n",
 		       ptr->filename);
