@@ -147,6 +147,10 @@ extern void named_section		PROTO((tree, char *, int));
 /* Tell assembler to switch to the section for function DECL.  */
 extern void function_section		PROTO((tree));
 
+/* Optionally record the information that a function has been defined
+   in this file */
+extern void mark_fn_defined_in_this_file (tree decl);
+
 /* Tell assembler to switch to the section for the exception table.  */
 extern void exception_section		PROTO((void));
 
